/*
 * RSSBot
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.rssbot.data.database.repository

import net.moonleay.rssbot.data.database.entry.SubscriptionData
import net.moonleay.rssbot.data.database.tables.SubscriptionsTable
import net.moonleay.rssbot.util.EmbedUtil
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.transactions.transaction

object SubscriptionRepository {

    fun getAll(): List<SubscriptionData> {
        val dataList = mutableListOf<SubscriptionData>()
        transaction {
            SubscriptionsTable.selectAll().forEach {
                dataList.add(
                    SubscriptionData(
                        it[SubscriptionsTable.id],
                        it[SubscriptionsTable.serverId],
                        it[SubscriptionsTable.channelId],
                        EmbedUtil.getColorFromString(it[SubscriptionsTable.subscriptionColor]),
                        it[SubscriptionsTable.subscriptionName],
                        it[SubscriptionsTable.feedUrl]
                    )
                )
            }
        }
        return dataList
    }

    fun getAllChannel(channelId: Long): List<SubscriptionData> {
        val dataList = mutableListOf<SubscriptionData>()
        transaction {
            SubscriptionsTable.select {
                SubscriptionsTable.channelId eq channelId
            }.forEach {
                dataList.add(
                    SubscriptionData(
                        it[SubscriptionsTable.id],
                        it[SubscriptionsTable.serverId],
                        it[SubscriptionsTable.channelId],
                        EmbedUtil.getColorFromString(it[SubscriptionsTable.subscriptionColor]),
                        it[SubscriptionsTable.subscriptionName],
                        it[SubscriptionsTable.feedUrl]
                    )
                )
            }
        }
        return dataList
    }


    fun get(channelId: Long, feedName: String): SubscriptionData? {
        var data: SubscriptionData? = null
        transaction {
            SubscriptionsTable.select {
                (SubscriptionsTable.channelId eq channelId) and (SubscriptionsTable.subscriptionName eq feedName)
            }.forEach {
                data = SubscriptionData(
                    it[SubscriptionsTable.id],
                    it[SubscriptionsTable.serverId],
                    it[SubscriptionsTable.channelId],
                    EmbedUtil.getColorFromString(it[SubscriptionsTable.subscriptionColor]),
                    it[SubscriptionsTable.subscriptionName],
                    it[SubscriptionsTable.feedUrl]
                )
            }
        }
        return data
    }

    fun existsWithUrl(channelId: Long, feedUrl: String): Boolean {
        var exists = false
        transaction {
            exists = SubscriptionsTable.select {
                (SubscriptionsTable.channelId eq channelId) and (SubscriptionsTable.feedUrl eq feedUrl)
            }.count() > 0
        }
        return exists
    }

    fun exists(channelId: Long, feedName: String): Boolean {
        var exists = false
        transaction {
            exists = SubscriptionsTable.select {
                (SubscriptionsTable.channelId eq channelId) and (SubscriptionsTable.subscriptionName eq feedName)
            }.count() > 0
        }
        return exists
    }


    fun write(data: SubscriptionData) = transaction {
        SubscriptionsTable.insert {
            it[serverId] = data.serverId
            it[channelId] = data.channelId
            it[subscriptionColor] = data.feedColor.readableName
            it[subscriptionName] = data.feedName
            it[feedUrl] = data.feedUrl
        }
    }

    fun delete(channelId: Long, feedName: String) = transaction {
        SubscriptionsTable.deleteWhere {
            (SubscriptionsTable.channelId eq channelId) and (SubscriptionsTable.subscriptionName eq feedName)
        }
    }
}
