/*
 * RSSBot
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.rssbot.extensions

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.commands.converters.impl.string
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.ephemeralSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import net.moonleay.rssbot.data.database.repository.RSSRepository
import net.moonleay.rssbot.data.database.repository.SubscriptionRepository
import net.moonleay.rssbot.util.EmbedColor
import net.moonleay.rssbot.util.MessageUtil

class StuffExtension : Extension() {

    override val name = "stuff"
    override val allowApplicationCommandInDMs: Boolean
        get() = false


    override suspend fun setup() {
        ephemeralSlashCommand(::StuffArguments) {
            name = "stuff"
            description = "Mark a feed for this channel as stuffed"
            this.action {
                val feedName = arguments.feedName
                val user = this.user.asUser()
                if (feedName == null) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "Missing Arguments",
                                "You are missing one or more arguments",
                                user.username
                            )
                        )
                    }
                    return@action
                }
                if (!SubscriptionRepository.exists(this.channel.id.value.toLong(), feedName)) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "Feed not found",
                                "The feed \"${feedName}\" was not found",
                                user.username
                            )
                        )
                    }
                    return@action
                }
                val subscription = SubscriptionRepository.get(this.channel.id.value.toLong(), feedName)
                RSSRepository.delete(subscription!!.id)
                SubscriptionRepository.delete(this.channel.id.value.toLong(), feedName)

                this.respond {
                    this.embeds.add(
                        MessageUtil.getEmbed(
                            EmbedColor.INFO,
                            "Removed Feed \"${feedName}\"",
                            "The feed \"${feedName}\" has been removed from this channel",
                            user.username
                        )
                    )
                }
            }
        }
    }

    inner class StuffArguments : Arguments() {
        val feedName by string {
            this.name = "name"
            this.description = "The name of the feed"
        }
    }
}
